// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

//
// This file was autogenerated by a tool.
// Do not modify it.
//

namespace Microsoft.Azure.Batch
{
    using Models = Microsoft.Azure.Batch.Protocol.Models;
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Information about an Azure Batch subtask.
    /// </summary>
    public partial class SubtaskInformation : IPropertyMetadata
    {
        private readonly ComputeNodeInformation computeNodeInformation;
        private readonly DateTime? endTime;
        private readonly int? exitCode;
        private readonly int? id;
        private readonly Common.TaskState? previousState;
        private readonly DateTime? previousStateTransitionTime;
        private readonly TaskSchedulingError schedulingError;
        private readonly DateTime? startTime;
        private readonly Common.TaskState? state;
        private readonly DateTime? stateTransitionTime;

        #region Constructors

        internal SubtaskInformation(Models.SubtaskInformation protocolObject)
        {
            this.computeNodeInformation = UtilitiesInternal.CreateObjectWithNullCheck(protocolObject.NodeInfo, o => new ComputeNodeInformation(o).Freeze());
            this.endTime = protocolObject.EndTime;
            this.exitCode = protocolObject.ExitCode;
            this.id = protocolObject.Id;
            this.previousState = UtilitiesInternal.MapNullableEnum<Models.TaskState, Common.TaskState>(protocolObject.PreviousState);
            this.previousStateTransitionTime = protocolObject.PreviousStateTransitionTime;
            this.schedulingError = UtilitiesInternal.CreateObjectWithNullCheck(protocolObject.SchedulingError, o => new TaskSchedulingError(o).Freeze());
            this.startTime = protocolObject.StartTime;
            this.state = UtilitiesInternal.MapNullableEnum<Models.TaskState, Common.TaskState>(protocolObject.State);
            this.stateTransitionTime = protocolObject.StateTransitionTime;
        }

        #endregion Constructors

        #region SubtaskInformation

        /// <summary>
        /// Gets the information about the compute node on which the subtask ran.
        /// </summary>
        public ComputeNodeInformation ComputeNodeInformation
        {
            get { return this.computeNodeInformation; }
        }

        /// <summary>
        /// Gets the time at which the subtask completed. This property is set only if the subtask is in the Completed state.
        /// </summary>
        public DateTime? EndTime
        {
            get { return this.endTime; }
        }

        /// <summary>
        /// Gets the exit code of the program specified on the subtask command line.
        /// </summary>
        /// <remarks>
        /// This property is only returned if the subtask is in the <see cref="Common.TaskState.Completed"/> state. The exit 
        /// code for a process reflects the specific convention implemented by the application developer for that process. 
        /// If you use the exit code value to make decisions in your code, be sure that you know the exit code convention 
        /// used by the application process. Note that the exit code may also be generated by the compute node operating 
        /// system, such as when a process is forcibly terminated.
        /// </remarks>
        public int? ExitCode
        {
            get { return this.exitCode; }
        }

        /// <summary>
        /// Gets the id of the subtask.
        /// </summary>
        public int? Id
        {
            get { return this.id; }
        }

        /// <summary>
        /// Gets the previous state of the subtask. This property is not set if the subtask is in its initial active state.
        /// </summary>
        public Common.TaskState? PreviousState
        {
            get { return this.previousState; }
        }

        /// <summary>
        /// Gets the time at which the subtask entered its previous state. This property is not set if the subtask is in 
        /// its initial active state.
        /// </summary>
        public DateTime? PreviousStateTransitionTime
        {
            get { return this.previousStateTransitionTime; }
        }

        /// <summary>
        /// Gets the details of any error encountered scheduling the subtask.
        /// </summary>
        public TaskSchedulingError SchedulingError
        {
            get { return this.schedulingError; }
        }

        /// <summary>
        /// Gets the time at which the subtask started running. If the subtask has been restarted or retried, this is the 
        /// most recent time at which the subtask started running.
        /// </summary>
        public DateTime? StartTime
        {
            get { return this.startTime; }
        }

        /// <summary>
        /// Gets the current state of the subtask.
        /// </summary>
        public Common.TaskState? State
        {
            get { return this.state; }
        }

        /// <summary>
        /// Gets the time at which the subtask entered its current state.
        /// </summary>
        public DateTime? StateTransitionTime
        {
            get { return this.stateTransitionTime; }
        }

        #endregion // SubtaskInformation

        #region IPropertyMetadata

        bool IModifiable.HasBeenModified
        {
            //This class is compile time readonly so it cannot have been modified
            get { return false; }
        }

        bool IReadOnly.IsReadOnly
        {
            get { return true; }
            set
            {
                // This class is compile time readonly already
            }
        }

        #endregion // IPropertyMetadata
    }
}